
/**
 * Modulo: biblioteca.c
 * 
 * Descripcion: contiene la definicion de las estructuras de datos y funciones  
 * primitivas de Biblioteca.
 *
 * Autor: Ivan Cantador, ivan.cantador@uam.es
 *
 * Version: 1.0
 * Fecha: 20/11/2017
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// ============================================================================
// VALORES CONSTANTES
// ============================================================================

#define MAX_LIBROS 256

#define OK 0
#define ERROR -1

// ============================================================================
// ESTRUCTURAS DE DATOS
// ============================================================================

typedef struct {
  char *titulo;
  char *autor;
  int anio;
} Libro;

typedef struct {
  Libro *libros[MAX_LIBROS];
  int numLibros;
} Biblioteca;


// ============================================================================
// FUNCIONES PRIMITIVAS DE LA ESTRUCTURA LIBRO
// ============================================================================

Libro *libro_crear(const char *tituloLibro, const char *autorLibro, int anioLibro) {
  Libro *pl = NULL;
  
  if ( !tituloLibro || !autorLibro || anioLibro <= 0  ) {
    return NULL;
  }
  
  pl = (Libro *) malloc(sizeof(Libro));
  if ( !pl ) {
    return NULL;
  }
  
  pl->titulo = (char *) malloc((1 + strlen(tituloLibro)) * sizeof(char));
  if ( !pl->titulo ) {
    free(pl);
    return NULL;
  }
  
  pl->autor = (char *) malloc((1 + strlen(autorLibro)) * sizeof(char));
  if ( !pl->autor ) {
    free(pl->titulo);
    free(pl);
    return NULL;
  }
  
  strcpy(pl->titulo, tituloLibro);
  strcpy(pl->autor, autorLibro);
  pl->anio = anioLibro;
  
  return pl;
}

void libro_liberar(Libro *pl) {
  if ( pl ) {
    if ( pl->titulo ) free(pl->titulo);
    if ( pl->autor ) free(pl->autor);
    free(pl);
  }
}

void libro_visualizar(Libro *pl) {
  if ( !pl || !pl->titulo || !pl->autor ) {
    return;
  }
  
  printf("titulo=%s|autor=%s|anio=%d", pl->titulo, pl->autor, pl->anio);
}

// ============================================================================
// FUNCIONES PRIMITIVAS DE LA ESTRUCTURA BIBLIOTECA
// ============================================================================

Biblioteca *biblioteca_crear() {
  Biblioteca *pb = NULL;
  
  pb = (Biblioteca *) malloc(sizeof(Biblioteca));
  if ( !pb ) {
    return NULL;
  }  
  
  pb->numLibros = 0;
  
  return pb;
}

int biblioteca_insertarLibro(Biblioteca *pb, const char *tituloLibro, const char *autorLibro, int anioLibro) {
  int ultimo;
  
  if ( !pb || !tituloLibro || !autorLibro || anioLibro <= 0  ) {
    return ERROR;
  }
  
  if ( pb->numLibros == MAX_LIBROS ) {
    return ERROR;
  }
  
  ultimo = pb->numLibros;
  
  pb->libros[ultimo] = libro_crear(tituloLibro, autorLibro, anioLibro);
  if ( !pb->libros[ultimo] ) {
    return ERROR;
  }
  
  pb->numLibros++;
  
  return OK;
}

void biblioteca_liberar(Biblioteca *pb) {
  int i;
  
  if ( pb ) {
    for ( i = 0; i < pb->numLibros; i++ ) {
      libro_liberar(pb->libros[i]);
    }
    free(pb);
  }
}

void biblioteca_visualizar(Biblioteca *pb) {
  int i;
  
  if ( pb ) {
    for ( i=0; i<pb->numLibros; i++ ) {
      printf("\tLibro %d: ", i);
      libro_visualizar(pb->libros[i]);
      printf("\n");
    }
  }
}

// ============================================================================
// PROGRAMA DE PRUEBA
// ============================================================================

int main() {
  Biblioteca *biblio = NULL;
  char cadena[5], cadena1[64], cadena2[64];
  int numero, status;
  
  biblio = biblioteca_crear();
  if ( !biblio ) {
    printf("Error: no se pudo crear la biblioteca.\n");
    return -1;
  }
  
  printf("Introduce el titulo del libro a insertar: ");
  gets(cadena1);
  
  printf("Introduce el autor del libro a insertar: ");
  gets(cadena2);
  
  printf("Introduce el anio del libro a insertar: ");
  gets(cadena);
  numero = atoi(cadena);
  
  status = biblioteca_insertarLibro(biblio, cadena1, cadena2, numero);
  if ( status == ERROR ) {
    printf("Error: no se pudo insertar el libro.\n");
    biblioteca_liberar(biblio);
    return -1;
  }
  

  printf("\nEstado de la biblioteca:\n");
  biblioteca_visualizar(biblio);

  biblioteca_liberar(biblio);

  return 1;
}
